import loadShapes from '../shapes/loadShapes';
import * as _ from 'lodash';
import * as EventUtils from '../utils/EventUtils';
import * as DataSetUtils from '../utils/DataSetUtils';
import * as setCoordConfig from '../components/setCoordConfig';
import * as setAxisConfig from '../components/setAxisConfig';
import * as setSeriesConfig from '../components/setSeriesConfig';
import * as setLengendConfig from '../components/setLengendConfig';
import * as setGuideConfig from '../components/setGuideConfig';
import * as setTooltipConfig from '../components/setTooltipConfig';
import * as setScaleConfig from '../components/setScaleConfig';
var G2 = require('@antv/g2');
var CommonChart = (function () {
    function CommonChart(config) {
        this.viewInstance = {};
        this.mainDataSet = {};
        this.config = _.cloneDeep(config);
        this.config = this.checkChartConfig(this.config);
        var chart = this.chartInstance = new G2.Chart(this.config.chart);
    }
    CommonChart.prototype.checkChartConfig = function (config) {
        var chart = config.chart;
        if (!chart || !chart.height) {
            throw new Error('please set correct chart option');
        }
        return config;
    };
    ;
    CommonChart.prototype.destroy = function (chart) {
        chart && chart.destroy();
    };
    CommonChart.prototype.clear = function (chart) {
        chart && chart.clear();
    };
    CommonChart.prototype.createView = function (chart, config) {
        var view = chart.view();
        if (!config.viewId) {
            throw new Error('you must set viewId');
        }
        this.viewInstance[config.viewId] = view;
        return view;
    };
    CommonChart.prototype.setEvents = function (chart, config) {
        EventUtils.setEvent(chart, null, config.chart);
    };
    CommonChart.prototype.setDataSource = function (chart, data) {
        chart.source(data);
    };
    CommonChart.prototype.setScale = function (chart, config) {
        return setScaleConfig.process(chart, config);
    };
    CommonChart.prototype.setCoord = function (chart, config) {
        return setCoordConfig.process(chart, config.coord);
    };
    CommonChart.prototype.setSeries = function (chart, config) {
        return setSeriesConfig.process(chart, config);
    };
    CommonChart.prototype.setAxis = function (chart, config) {
        return setAxisConfig.process(chart, config);
    };
    CommonChart.prototype.setTooltip = function (chart, config) {
        return setTooltipConfig.process(chart, config);
    };
    CommonChart.prototype.setGuide = function (chart, config) {
        return setGuideConfig.process(chart, config);
    };
    CommonChart.prototype.setLegend = function (chart, config) {
        return setLengendConfig.process(chart, config);
    };
    CommonChart.prototype.setContent = function (chart, config) {
        this.setScale(chart, config);
        this.setAxis(chart, config);
        this.setSeries(chart, config);
        this.setCoord(chart, config);
        this.setTooltip(chart, config);
        this.setGuide(chart, config);
    };
    CommonChart.prototype.setFacetContent = function (chart, config) {
        this.setScale(chart, config);
        this.setSeries(chart, config);
        this.setGuide(chart, config);
    };
    CommonChart.prototype.setView = function (item, chart, config) {
        var view = this.createView(chart, item);
        var viewData = item.data;
        if (item.data) {
            viewData = DataSetUtils.getProcessedData(item.data, item.dataPre);
        }
        else if (!item.data && item.dataPre) {
            viewData = DataSetUtils.getProcessedData(config.data, item.dataPre);
        }
        else if (!item.data && !item.dataPre) {
            viewData = this.mainDataSet;
        }
        var calData = DataSetUtils.getDataContent(viewData, item.dataView);
        this.setDataSource(view, calData);
        this.setContent(view, item);
        return view;
    };
    CommonChart.prototype.setViews = function (chart, config) {
        var views = config.views;
        if (_.isEmpty(views)) {
            return;
        }
        views = Array.isArray(views) ? views : [views];
        for (var _i = 0, views_1 = views; _i < views_1.length; _i++) {
            var item = views_1[_i];
            this.setView(item, chart, config);
        }
    };
    CommonChart.prototype.setFacetViews = function (chart, facet, views) {
        var viewData = DataSetUtils.getProcessedData(facet.data, views.dataPre);
        var calData = DataSetUtils.getDataContent(viewData, views.dataView);
        this.setDataSource(chart, calData);
        this.setFacetContent(chart, views);
    };
    CommonChart.prototype.setFacet = function (chart, config) {
        var _this = this;
        var facet = config.facet;
        if (!facet) {
            return;
        }
        var options = _.omit(facet, ['type', 'views']);
        if (_.isEmpty(facet.views) && !_.isFunction(facet.views)) {
            return chart.facet(facet.type, options);
        }
        if (_.isFunction(facet.views)) {
            options.eachView = function (v, f) {
                var options = facet.views(v, f);
                _this.setFacetViews(v, f, options);
            };
        }
        else {
            facet.views = Array.isArray(facet.views) ? facet.views : [facet.views];
            options.eachView = function (v, f) {
                _this.setFacetViews(v, f, facet.views[0]);
            };
        }
        return chart.facet(facet.type, options);
    };
    CommonChart.prototype.render = function () {
        var config = this.config;
        var data = config.data, dataPre = config.dataPre, dataView = config.dataView;
        var chart = this.chartInstance;
        loadShapes();
        this.setEvents(chart, config);
        var chartData = this.mainDataSet = DataSetUtils.getProcessedData(data, dataPre);
        if (config.series || config.facet) {
            var calData = DataSetUtils.getDataContent(chartData, dataView);
            this.setDataSource(chart, calData);
        }
        this.setContent(chart, config);
        this.setLegend(chart, config);
        this.setViews(chart, config);
        this.setFacet(chart, config);
        this.oriConfig = config;
        chart.render();
    };
    CommonChart.prototype.repaintWidthHeight = function (config) {
        var oriConfig = this.oriConfig;
        var chart = this.chartInstance;
        var width = _.get(config, 'chart.width');
        if (width && !_.isEqual(_.get(oriConfig, 'chart.width'), width)) {
            chart.changeWidth(width);
        }
        var height = _.get(config, 'chart.height');
        if (height && !_.isEqual(_.get(oriConfig, 'chart.height'), height)) {
            chart.changeHeight(height);
        }
    };
    CommonChart.prototype.repaintData = function (chart, oriConfig, config) {
        var hasDataChange = false;
        if (!_.isEmpty(config.data) && !_.isEqual(oriConfig.data, config.data)) {
            var viewData = DataSetUtils.getProcessedData(config.data, config.dataPre);
            var calData = DataSetUtils.getDataContent(viewData, config.dataView);
            hasDataChange = true;
            chart.changeData(calData);
        }
        return hasDataChange;
    };
    CommonChart.prototype.repaintViewData = function (chart, oriConfig, config) {
        var viewData;
        if ((_.isEmpty(config.data) && !_.isEmpty(oriConfig.data)) ||
            (!_.isEmpty(config.data) && !_.isEqual(oriConfig.data, config.data))) {
            viewData = DataSetUtils.getProcessedData(config.data, config.dataPre);
            var calData = DataSetUtils.getDataContent(viewData, config.dataView);
            chart.changeData(calData);
        }
    };
    CommonChart.prototype.repaintContent = function (chart, oriConfig, config) {
        var hasChartChange = false;
        if (config.scale && !_.isEqual(oriConfig.scale, config.scale)) {
            this.setScale(chart, config);
            hasChartChange = true;
        }
        if (config.coord && !_.isEqual(oriConfig.coord, config.coord)) {
            this.setCoord(chart, config);
            hasChartChange = true;
        }
        if (config.axis && !_.isEqual(oriConfig.axis, config.axis)) {
            this.setAxis(chart, config);
            hasChartChange = true;
        }
        if (config.series && !_.isEqual(oriConfig.series, config.series)) {
            this.setSeries(chart, config);
            hasChartChange = true;
        }
        if (config.tooltip && !_.isEqual(oriConfig.tooltip, config.tooltip)) {
            this.setTooltip(chart, config);
            hasChartChange = true;
        }
        if (config.guide && !_.isEqual(oriConfig.guide, config.guide)) {
            this.setGuide(chart, config);
            hasChartChange = true;
        }
        return hasChartChange;
    };
    CommonChart.prototype.repaintViews = function (chart, oriConfig, config) {
        var viewsConfig = config.views;
        var oViewsConfig = oriConfig.views;
        if (!_.isEqual(oriConfig.views, config.views)) {
            var _loop_1 = function (item) {
                var oriView = oViewsConfig.filter(function (res) { return (res.viewId === item.viewId); });
                var view = void 0;
                if (oriView.length) {
                    view = this_1.viewInstance[item.viewId];
                    this_1.repaintViewData(view, oriView[0], item);
                    this_1.repaintContent(view, oriView[0], item);
                }
                else {
                    view = this_1.setView(item, chart, config);
                }
                view.repaint();
            };
            var this_1 = this;
            for (var _i = 0, viewsConfig_1 = viewsConfig; _i < viewsConfig_1.length; _i++) {
                var item = viewsConfig_1[_i];
                _loop_1(item);
            }
        }
    };
    CommonChart.prototype.renderDiffConfig = function (config) {
        var oriConfig = this.oriConfig;
        var viewsConfig = config.views;
        var chart = this.chartInstance;
        config = this.checkChartConfig(config);
        this.repaintWidthHeight(config);
        var hasDataChange = this.repaintData(chart, oriConfig, config);
        if (hasDataChange) {
            this.mainDataSet = config.data;
        }
        var hasContentChange = this.repaintContent(chart, oriConfig, config);
        this.repaintViews(chart, oriConfig, config);
        var hasChartPartChange = false;
        if (config.legend && !_.isEqual(oriConfig.legend, config.legend)) {
            this.setLegend(chart, config);
            hasChartPartChange = true;
        }
        if (config.facet && !_.isEqual(oriConfig.facet, config.facet)) {
            this.setFacet(chart, config);
            hasChartPartChange = true;
        }
        if (hasContentChange || hasChartPartChange) {
            chart.repaint();
        }
    };
    CommonChart.prototype.repaint = function (config) {
        if (_.isEmpty(config)) {
            return;
        }
        config = _.cloneDeep(config);
        this.renderDiffConfig(config);
        this.oriConfig = config;
    };
    CommonChart.prototype.getWidth = function () {
        return this.chartInstance.get('width');
    };
    CommonChart.prototype.getHeight = function () {
        return this.chartInstance.get('height');
    };
    return CommonChart;
}());
export default CommonChart;
//# sourceMappingURL=CommonChart.js.map